import { EventBarModes } from "../../interfaces/frontend"
import Item from "./Item"
import { GameSettings } from "@components/Lobby/SettingsFrame/Setting"
import {
  faSquare2,
  faSquare3,
  faSquare4,
} from "@fortawesome/pro-regular-svg-icons"
import {
  faArrowRightFromBracket,
  faBroomWide,
  faCheck,
  faComments,
  faEye,
  faEyeSlash,
  faGlasses,
  faPalette,
  faReply,
  faRotate,
  faScribble,
  faShip,
  faSparkles,
  faSwords,
} from "@fortawesome/pro-solid-svg-icons"
import { useDrawProps } from "@hooks/useDrawProps"
import { useGameProps } from "@hooks/useGameProps"
import { socket } from "@lib/socket"
import { GamePropsSchema } from "@lib/zodSchemas"
import { Orientation } from "@prisma/client"
import { useSession } from "next-auth/react"
import {
  Dispatch,
  SetStateAction,
  useCallback,
  useEffect,
  useMemo,
} from "react"

export function setGameSetting(
  payload: GameSettings,
  setSetting: (settings: GameSettings) => string | null,
  full: (payload: GamePropsSchema) => void
) {
  return () => {
    const hash = setSetting(payload)
    socket.emit("gameSetting", payload, (newHash) => {
      if (newHash === hash) return
      console.log("hash", hash, newHash)
      socket.emit("update", full)
    })
  }
}

function EventBar({ clear }: { clear: () => void }) {
  const { shouldHide, color } = useDrawProps()
  const { data: session } = useSession()
  const {
    ships,
    payload,
    menu,
    mode,
    setSetting,
    full,
    setTarget,
    setTargetPreview,
    setIsReady,
  } = useGameProps()
  const gameSetting = useCallback(
    (payload: GameSettings) => setGameSetting(payload, setSetting, full),
    [full, setSetting]
  )
  const self = useMemo(
    () => payload?.users.find((e) => e?.id === session?.user.id),
    [payload?.users, session?.user.id]
  )
  const items = useMemo<EventBarModes>(
    () => ({
      main: [
        {
          icon: "burger-menu",
          text: "Menu",
          callback: () => {
            useGameProps.setState({ menu: "menu" })
          },
        },
        payload?.game?.state === "running"
          ? {
              icon: faSwords,
              text: "Attack",
              callback: () => {
                useGameProps.setState({ menu: "actions" })
              },
            }
          : {
              icon: faShip,
              text: "Ships",
              callback: () => {
                useGameProps.setState({ menu: "actions" })
              },
            },
        {
          icon: "pen",
          text: "Draw",
          callback: () => {
            useGameProps.setState({ menu: "draw" })
          },
        },
        {
          icon: "gear",
          text: "Settings",
          callback: () => {
            useGameProps.setState({ menu: "settings" })
          },
        },
      ],
      menu: [
        {
          icon: faArrowRightFromBracket,
          text: "Leave",
          iconColor: "darkred",
          callback: () => {
            // router.push()
          },
        },
      ],
      actions:
        payload?.game?.state === "running"
          ? [
              {
                icon: "scope",
                text: "Fire missile",
                callback: () => {
                  useGameProps.setState({ mode: 3 })
                  setTarget((e) => ({ ...e, show: false }))
                },
              },
              {
                icon: "torpedo",
                text: "Fire torpedo",
                amount:
                  2 -
                  (self?.moves.filter(
                    (e) => e.action === "htorpedo" || e.action === "vtorpedo"
                  ).length ?? 0),
                callback: () => {
                  useGameProps.setState({ mode: 1 })
                  setTarget((e) => ({ ...e, show: false }))
                },
              },
              {
                icon: "radar",
                text: "Radar scan",
                amount:
                  1 -
                  (self?.moves.filter((e) => e.action === "radar").length ?? 0),
                callback: () => {
                  useGameProps.setState({ mode: 0 })
                  setTarget((e) => ({ ...e, show: false }))
                },
              },
            ]
          : [
              {
                icon: faSquare2,
                text: "Minensucher",
                amount: 1 - ships.filter((e) => e.size === 2).length,
                callback: () => {
                  if (1 - ships.filter((e) => e.size === 2).length === 0) return
                  useGameProps.setState({ mode: 0 })
                },
              },
              {
                icon: faSquare3,
                text: "Kreuzer",
                amount: 3 - ships.filter((e) => e.size === 3).length,
                callback: () => {
                  if (3 - ships.filter((e) => e.size === 3).length === 0) return
                  useGameProps.setState({ mode: 1 })
                },
              },
              {
                icon: faSquare4,
                text: "Schlachtschiff",
                amount: 2 - ships.filter((e) => e.size === 4).length,
                callback: () => {
                  if (2 - ships.filter((e) => e.size === 4).length === 0) return
                  useGameProps.setState({ mode: 2 })
                },
              },
              {
                icon: faRotate,
                text: "Rotate",
                callback: () => {
                  setTargetPreview((t) => ({
                    ...t,
                    orientation: t.orientation === "h" ? "v" : "h",
                  }))
                },
              },
              {
                icon: faCheck,
                text: "Done",
                disabled: mode >= 0,
                callback: () => {
                  const i = payload?.users.findIndex(
                    (user) => session?.user?.id === user?.id
                  )
                  if (!i) return
                  setIsReady({ isReady: true, i })
                },
              },
            ],
      draw: [
        { icon: faBroomWide, text: "Clear", callback: clear },
        { icon: faPalette, text: "Color", iconColor: color },
        {
          icon: shouldHide ? faEye : faEyeSlash,
          text: shouldHide ? "Show" : "Hide",
          callback: () => {
            useDrawProps.setState({ shouldHide: !shouldHide })
          },
        },
      ],
      settings: [
        {
          icon: faGlasses,
          text: "Spectators",
          disabled: !payload?.game?.allowSpectators,
          callback: gameSetting({
            allowSpectators: !payload?.game?.allowSpectators,
          }),
        },
        {
          icon: faSparkles,
          text: "Specials",
          disabled: !payload?.game?.allowSpecials,
          callback: gameSetting({
            allowSpecials: !payload?.game?.allowSpecials,
          }),
        },
        {
          icon: faComments,
          text: "Chat",
          disabled: !payload?.game?.allowChat,
          callback: gameSetting({ allowChat: !payload?.game?.allowChat }),
        },
        {
          icon: faScribble,
          text: "Mark/Draw",
          disabled: !payload?.game?.allowMarkDraw,
          callback: gameSetting({
            allowMarkDraw: !payload?.game?.allowMarkDraw,
          }),
        },
      ],
    }),
    [
      clear,
      color,
      gameSetting,
      mode,
      payload?.game?.allowChat,
      payload?.game?.allowMarkDraw,
      payload?.game?.allowSpecials,
      payload?.game?.allowSpectators,
      payload?.game?.state,
      payload?.users,
      self?.moves,
      session?.user?.id,
      setIsReady,
      setTarget,
      setTargetPreview,
      ships,
      shouldHide,
    ]
  )

  useEffect(() => {
    if (
      menu !== "actions" ||
      payload?.game?.state !== "starting" ||
      mode < 0 ||
      items.actions[mode].amount
    )
      return
    const index = items.actions.findIndex((e) => e.amount)
    useGameProps.setState({ mode: index })
  }, [items.actions, menu, mode, payload?.game?.state])

  useEffect(() => {
    useDrawProps.setState({ enable: menu === "draw" })
  }, [menu])

  return (
    <div className="event-bar">
      {menu !== "main" && (
        <Item
          props={{
            icon: faReply,
            text: "Return",
            iconColor: "#555",
            callback: () => {
              useGameProps.setState({ menu: "main" })
            },
          }}
        ></Item>
      )}
      {items[menu].map((e, i) => (
        <Item key={i} props={e} />
      ))}
    </div>
  )
}

export default EventBar
