// import Bluetooth from "./Bluetooth"
// import FogImages from "./FogImages"
import Labeling from "./Labeling"
import Ships from "./Ships"
import BorderTiles from "@components/Gamefield/BorderTiles"
import EventBar from "@components/Gamefield/EventBar"
import HitElems from "@components/Gamefield/HitElems"
import Targets from "@components/Gamefield/Targets"
import { useDraw } from "@hooks/useDraw"
import { useDrawProps } from "@hooks/useDrawProps"
import { useGameProps } from "@hooks/useGameProps"
import useSocket from "@hooks/useSocket"
import { socket } from "@lib/socket"
import { overlapsWithAnyBorder } from "@lib/utils/helpers"
import { CSSProperties } from "react"
import { useEffect } from "react"

export const count = 12

function Gamefield() {
  const {
    mode,
    target,
    targetPreview,
    mouseCursor,
    ships,
    payload,
    setTargetPreview,
    full,
  } = useGameProps()
  const { isConnected } = useSocket()

  useEffect(() => {
    if (payload?.game?.id || !isConnected) return
    socket.emit("update", full)
  }, [full, payload?.game?.id, isConnected])

  useEffect(() => {
    if (mode < 0) return
    const { x, y, show } = target
    const { shouldShow, ...position } = mouseCursor
    if (
      !shouldShow ||
      (payload?.game?.state === "running" &&
        overlapsWithAnyBorder(position, mode))
    )
      setTargetPreview((t) => ({ ...t, show: false }))
    else {
      setTargetPreview((t) => ({
        ...t,
        ...position,
        show: !show || x !== position.x || y !== position.y,
      }))
      if (payload?.game?.state !== "starting") return
      const handleKeyPress = (event: KeyboardEvent) => {
        if (event.key !== "r") return
        setTargetPreview((t) => ({
          ...t,
          orientation: t.orientation === "h" ? "v" : "h",
        }))
      }
      document.addEventListener("keydown", handleKeyPress)
      return () => {
        document.removeEventListener("keydown", handleKeyPress)
      }
    }
  }, [mode, mouseCursor, payload?.game?.state, setTargetPreview, target])

  const { canvasRef, onMouseDown, clear } = useDraw()
  const { enable, color, shouldHide } = useDrawProps()

  return (
    <div id="gamefield">
      {/* <Bluetooth /> */}
      <div
        id="game-frame"
        style={{ "--i": count } as CSSProperties}
        // onMouseLeave={() =>
        //   setMouseCursor((e) => ({ ...e, shouldShow: false }))
        // }
      >
        {/* Bordes */}
        <BorderTiles />

        {/* Collumn lettes and row numbers */}
        <Labeling />

        {/* Ships */}
        <Ships />
        <HitElems />

        {/* Fog images */}
        {/* <FogImages /> */}

        <Targets />

        <canvas
          style={
            {
              opacity: shouldHide ? 0 : 1,
              boxShadow: enable ? "inset 0 0 0 2px " + color : "none",
              pointerEvents: enable && !shouldHide ? "auto" : "none",
            } as CSSProperties
          }
          ref={canvasRef}
          onMouseDown={onMouseDown}
          width="648"
          height="648"
        />
      </div>
      <EventBar clear={clear} />
    </div>
  )
}

export default Gamefield
